from .models import FileModel, FileConfigModel,Column
from .runner import Runner
from .utils import readModel
from pydantic import ValidationError
import argparse
from os import getcwd

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-d', '--directory', help="specify a different directory to fetch config file", default=getcwd())
    args = parser.parse_args()

    try: 
        model = readModel(args.directory, 'parser.cfg')
        df = Runner(model)
        df.execute()

    except ValidationError as e:
        print(e)






from os import path
from pandas import read_json, read_csv
from .models import FileConfigModel, FileModel
from .algorithms import *

class Runner:

    def __init__(self, model: FileConfigModel):
        _file = FileModel(name=model.name, path=path.join(model.path, model.name))
        self.model = model
        self._verifyRows(_file.path)
        if model.file_type == 'json':
            self.dataframe = read_json(path_or_buf=_file.path, orient='index')
        elif model.file_type == 'csv':
            self.dataframe = read_csv(filepath_or_buffer=_file.path, sep=model.columns_delimiter)
        column_names = list(map(lambda x: x.name, model.columns))
        self.dataframe.columns = column_names

    def _verifyRows(self, file):
        with open(file, 'r') as csv:
            first_line = csv.readline()
        if first_line.count(self.model.columns_delimiter) + 1 != len(self.model.columns):
            raise Exception('Number of columns differ')


    def execute(self):
        for algo in self.model.algorithms:
            if algo.name == "delete":
                print(pseudo(self.dataframe, algo.options))
            else: raise NotImplementedError
        if self.model.export: self.save()

    def save(self):
        exporter = self.model.export_rules
        if exporter.output_format == 'csv':
            self.dataframe.to_csv(path_or_buf=exporter.output_name, sep=self.model.columns_delimiter, index=False)
        elif exporter.output_format == 'json':
            self.dataframe.to_json(exporter.output_name, index=False)
        else: raise NotImplementedError





from typing import List, Literal, Union
from numpy import NaN
from pydantic import BaseModel, DirectoryPath, FilePath, Field


class Column(BaseModel):
    name: str
    column_type: Literal['int', 'str', 'datetime'] 
    
class FileModel(BaseModel):
    name: str
    path: FilePath

class DeleteOptions(BaseModel):
    columns: List[int]
    alias: str = NaN

class DeleteAlgorithm(BaseModel):
    name: Literal['delete']
    options: DeleteOptions

class Exporter(BaseModel):
    output_name: str
    output_format: Literal['csv', 'json']
    
class FileConfigModel(BaseModel):
    name: str
    path: DirectoryPath
    columns: List[Column] = Field(..., description= "Columns of submitted file")
    columns_delimiter: Literal['\t', '\n'] = '\t'
    algorithms: List[Union[DeleteAlgorithm]]
    file_type: Literal['csv', 'json']
    export: bool = True
    export_rules: Exporter




from .models import FileModel, FileConfigModel
from os import path

def readModel(directory, name):
    configFile = FileModel(name=name, path=path.join(directory, name))
    model = FileConfigModel.parse_file(configFile.path)
    return model



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['insanonym_utils']

package_data = \
{'': ['*']}

install_requires = \
['numpy>=1.20.2,<2.0.0',
 'pandas>=1.2.4,<2.0.0',
 'pydantic>=1.8.1,<2.0.0',
 'pytest>=6.2.3,<7.0.0']

entry_points = \
{'console_scripts': ['anon = insanonym_utils.main:main']}

setup_kwargs = {
    'name': 'insanonym-utils',
    'version': '0.2.0',
    'description': 'Execute anonymization scripts over a table',
    'long_description': None,
    'author': 'Daniel Mathiot',
    'author_email': 'd.danymat@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
